use strict;
use warnings;

#################################################################################################################################
#Quantitation
##########################################################################################
sub spectrum_quantitator{
    my($spect,$par,$rep)=@_;
    my %spectra_hash=%{$spect};#mgf hash or input hash
    my %param=%{$par};#parameter hash
    my %reporter=%{$rep};#global plex hash
    my %rep_ion;#hash to handle reporter ion peaks. {113}{mz}=int
    my @peaks=@{$spectra_hash{MS2}};#save all ms2 peaks in array
    my %rem;#peaks other than reporters (valleys)
    my @max_peak;# keep max of reporter peaks to compare n remove valley peaks higher than this
    my %reporter_count;#{113}=freq;
    #for loop for all m/z intensities
    foreach my $report(@{$reporter{$param{Label_type}}{$param{Plex}}})
    {
        $rep_ion{$report}=undef;##assigning all reporter ion tags a priori so that missing ones can be put to 0
    }
    for(my $i=0; $i<@peaks;$i++)
    {
        my @temp=split/\s+/,$peaks[$i];
        foreach my $report(@{$reporter{$param{Label_type}}{$param{Plex}}})
        {
            
            #check if mass falls in that reporter region if yes then assign in hash and delete
	    if ($param{Tol_unit} eq "Da")
	    {
		if ($temp[0]>=$report-$param{Tol_min})
		{
		    if($temp[0]<=$report+$param{Tol_max})
		    {
		        $rep_ion{$report}{$temp[0]}=$temp[1];
		        $reporter_count{$report}=undef;
		        push(@max_peak,$temp[1]);
		        delete $peaks[$i];
		        last;
		    }
		}
            }
	    else#ppm
	    {
		if ($temp[0]>=$report-(($report*$param{Tol_min})/1000000))
		{
		    if($temp[0]<=$report+(($report*$param{Tol_max})/1000000))
		    {
		        $rep_ion{$report}{$temp[0]}=$temp[1];
		        $reporter_count{$report}=undef;
		        push(@max_peak,$temp[1]);
		        delete $peaks[$i];
		        last;
		    }
		}
	    }
        }
    }
    my $rep_count=scalar keys %reporter_count;
    if ($rep_count<$param{min_reporters})
    {   
        return($spect,\%rep_ion,$rep_count);
    }
    
    my @peaks_2;
    foreach my $values(@peaks)
    {
        if (!defined $values)
        {
            #leave empty
        }
        else
        {
            push(@peaks_2,$values);
        }
    }
    my ($min,$max)=(sort{$b<=>$a} @max_peak)[-1,0];
    #my $reference=shift @max_peak;
    if (scalar @peaks_2 >0)
    {
        #print "$peaks_2[0]";<>;
        foreach my $values(@peaks_2)
        {
            chomp $values;
            my @remain=split/\s+/,$values;
            #print "$remain[0]=$remain[1]\n";
            if ($remain[1]>$min)
            {
                next;
            }
            $rem{$remain[0]}=$remain[1];
        }
    }
    
        
    ##########################################################################################    
    ######################################################################################################################
    #calculate area by specified method and store spectrum results in a hash
    #####################################
    #till now all the masses are filtered in their repsective reporter keys in hash rep_ion
    #next check parameter in the param hash about spectra area calculation and calculate area based on that
    
    my $area_hash;#{T/B/S}{Area/Error/Peak_Max}
    my $area_cal={
        "T"=>\&area_T,
        "B"=>\&area_B,
        "S"=>\&area_S
                 };
    
    if ($param{Spectrum_area}=~m/T/ || $param{Spectrum_area}=~m/B/ || $param{Spectrum_area}=~m/S/)
    {
        #my $sub='area_'.$param{Spectrum_area};
        #returns a referenced hash containg first key is tag as in rep_ion, second key is "Area"/"Peak_Max"/"Error" and value is the results
        #in case input is TBS first key is area method T/B/S second is rep_ion and rest follows
        #&{ \&$name }();#find a better way to do it &$sub
        $area_hash=$area_cal->{$param{Spectrum_area}}->(\%rep_ion,\%rem,$area_hash);#\&$sub(\%rep_ion,\@peaks,$area_hash);####calling a subroutine by the name it codes   
    }
    else
    {
        print "Only T/S/B (T= trapezoid, S= sum intensity, B= baseline corrected sum intensity) methods are supported. Reverting to base line correction area method\n";
        #redirect to baseline method
        $area_hash=area_B(\%rep_ion,\%rem,$area_hash);
    }
    #find determinant for purity correction values
    #calculate purity corrected area
    $area_hash=isotopic_correction($area_hash,$param{Purity_corrections_values},$param{Plex});
    
    ##################how to do this part??
    #calculate normalized area if required
    if ($param{Normalization} eq "yes")
    {
        $area_hash=normalize($area_hash,$param{Spectrum_area});
    }
    ##########################################################################
    #area under threshold
    if (! defined($param{Peak_intensity_threshold}))
    {
        $param{Peak_intensity_threshold}=0;##made deafualt 0
    }
    $area_hash=intensity_cutoff($area_hash,$param{Peak_intensity_threshold},$param{Spectrum_area});
    ######################################################################################
    #calculate ratio and respective error and then print
    
    #Ratio_err calculates ratio and errors
    $area_hash=ratio_err($area_hash,$param{Base_reporter},$param{Spectrum_area});
    #print "ratio cal crossed";<>;
    my %area_has=%{$area_hash};
    #print keys %{$area_has{T}{'115.1'}},"\t",values %{$area_has{T}{'115.1'}};<>;
    ######################################################################################
    #print spectra
    ######################################################################################
    #$$this is happening for single spectra again and again
    
    return(\%spectra_hash,$area_hash,$rep_count);
    #print_spec(\%spectra_hash,$area_hash,$fh);  
}




sub print_spec{
    my ($spec_cnt,$spectra_det,$area_val,$fh)=@_;
    my %spec=%{$spectra_det};
    my %area_hash=%{$area_val};
    foreach my $type(sort keys %area_hash)
    {
        #print $spec{INT};<>;
        print $fh "$spec_cnt\t$spec{TITLE}\t$spec{RT}\t$spec{INT}\t$spec{MZ}\t$spec{Z}\t$spec{MSLevel}\t$spec{PrevScan}";
        foreach my $tag(sort keys %{$area_hash{$type}})
        {
            #print $fh "\t$area_hash{$type}{$tag}{Area}\t$area_hash{$type}{$tag}{Error}\t$area_hash{$type}{$tag}{Peak_Max}\t$area_hash{$type}{$tag}{Ratio}\t$area_hash{$type}{$tag}{Ratio_err}";
            print $fh "\t$area_hash{$type}{$tag}{Ratio}";
        }
        foreach my $tag(sort keys %{$area_hash{$type}})
        {
            print $fh "\t$area_hash{$type}{$tag}{Ratio_err}";
        }
        foreach my $tag(sort keys %{$area_hash{$type}})
        {
            print $fh "\t$area_hash{$type}{$tag}{Peak_Max}";
        }
        foreach my $tag(sort keys %{$area_hash{$type}})
        {
            print $fh "\t$area_hash{$type}{$tag}{Area}";
        }
         foreach my $tag(sort keys %{$area_hash{$type}})
        {
            print $fh "\t$area_hash{$type}{$tag}{Error}";
        }
        print $fh "\n";
    
    }
            
}

#####################################################################################
#subroutines for all three ::
#area calculations of spectrum
#area calculations of peptide
#area calculations of protein
#####################################################################################
#subroutines for:
#error calculation for area, combination and ratio
#ratio calculation
#determinant calculation
#isotopic corrections
#normalization
#round off
#####################################################################################
#mgf file parser
#mzML file parser
########################################################################################
#optional:
#run X!tandem for iTRAQ peptide identifications
#tandem XML parser
#result integration
#########################################################################################################

#########################################################################################################
#area calculations of spectrum
sub area_T{#trapezoid area calculation
    my($inp,$med,$area_hash)=@_;
    my %input=%{$inp};
    #we don't want noisy peaks array in here so we wud not deref $med
    my %output;
    foreach my $tag(sort keys(%input))
    {
        if (!defined $input{$tag})
        {
            
            $output{T}{$tag}{Area}=0;#NF=not found
            $output{T}{$tag}{Peak_Max}=0;
            $output{T}{$tag}{Error}=0;
            #print "$tag\t$output{T}{$tag}{Area}";<>;
            next;
        }
        #print "$tag";<>;
        my %hash=%{$input{$tag}};
        #my @MS2mz=keys(%hash);
        my $peakmax=(sort{$a<=>$b} values %hash)[-1];
        #print "$peakma_n";<>;
        my @MS2mz=sort {$a <=> $b} keys(%hash);
        my $area=0;
        #my $peakmax=0;
        my $total_int=0;#for error
        for(my $i=0;$i<@MS2mz-1;$i++)
        {
            my $intmz1=$hash{$MS2mz[$i]};
            my $intmz2=$hash{$MS2mz[$i+1]};
            my $height=($intmz1+$intmz2)/2;
            #print "$intmz1\t$intmz2";<>;
            my $width=$MS2mz[$i+1]-$MS2mz[$i];
            $area=$area+($height*$width);
            $total_int=$total_int+$intmz1;##for last int is not added
        }
        #print "area is :: $area";<>;
        if (scalar @MS2mz==1)
        {
            $area=$hash{$MS2mz[-1]};
        }
        
        $total_int+=$hash{$MS2mz[-1]};#last index falls off thats why add last intensity at last
        my $error=0;
        if ($peakmax>1)
        {
           $error=$peakmax**0.5;
        }
        elsif($peakmax==1)
        {
           $error=$total_int**0.5;
        }
        elsif($peakmax==0)
        {
           $error=0;
        }
        #print "$area\t$peakmax\t$error\n" if scalar @MS2mz ==1;<>;
        $output{T}{$tag}{Area}=$area;
        $output{T}{$tag}{Peak_Max}=$peakmax;
        $output{T}{$tag}{Error}=$error;
         #print "$tag\t$output{T}{$tag}{Area}";<>;
        
    }
    return(\%output);
    
}
#########################################################################################################
#baseline corrected area
sub area_B{
    my($inp,$med,$area_hash)=@_;
    my %input=%{$inp};
    #we want median from "noisy peaks array" so we wud deref $med
    my %rem=%{$med};
    my %output;
    #######################Calculate $median before calculating each tags area and error#######################
    my $median=0;
    
    if (scalar (keys %rem) >0)
    {
        my @rem1=sort values %rem;
        #print "remaining array=@rem1\n";
        my $mid=int @rem1/2;
        if (@rem1 % 2)#ODD
        {
           $median=$rem1[$mid];##array index starts from 0 so index shifts by -1
        }
        else#EVEN
        {
            $median=($rem1[$mid-1]+$rem1[$mid])/2;
        }   
    }
    else
    {
        $median=0;
    }
    #print $median;<>;
    
    ###########################################################################################################
    foreach my $tag(sort keys(%input))
    {
        if (!defined $input{$tag})
        {
            $output{B}{$tag}{Area}=0;#NF=not found
            $output{B}{$tag}{Peak_Max}=0;
            $output{B}{$tag}{Error}=0;
            next;
        }
        my %hash=%{$input{$tag}};
        #my @MS2mz=
        my @MS2mz=sort {$a <=> $b} keys(%hash);
        my $area=0;
        my $peakmax=0;
        for(my $i=0;$i<@MS2mz;$i++)
        {
            my $intmz1=$hash{$MS2mz[$i]};
            $area=$area+$intmz1;
            if ($intmz1 >$peakmax)
            {
                $peakmax=$intmz1;
            }
            
        }
        $area=$area-$median;
        my $error=0;
        if ($peakmax>1)
        {
           $error=$peakmax**0.5;
        }
        elsif($peakmax==1)
        {
           $error=$area**0.5;
        }
        elsif($peakmax==0)
        {
           $error=0;
        }
        $output{B}{$tag}{Area}=$area;
        $output{B}{$tag}{Peak_Max}=$peakmax;
        $output{B}{$tag}{Error}=$error;
        
        
    }
    #print "out of loop\n";<>;
    return(\%output);  
}
#########################################################################################################
#sumintensity area
sub area_S{
    my($inp,$med,$area_hash)=@_;
    my %input=%{$inp};
    my %output;
    foreach my $tag(sort keys(%input))
    {
        if (!defined $input{$tag})
        {
            $output{S}{$tag}{Area}=0;#NF=not found
            $output{S}{$tag}{Peak_Max}=0;
            $output{S}{$tag}{Error}=0;
            next;
        }
        my %hash=%{$input{$tag}};
        #my @MS2mz=
        my @MS2mz=sort {$a <=> $b} keys(%hash);
        my $area=0;
        my $peakmax=0;
        for(my $i=0;$i<@MS2mz;$i++)
        {
            my $intmz1=$hash{$MS2mz[$i]};
            $area=$area+$intmz1;
            if ($intmz1 >$peakmax)
            {
                $peakmax=$intmz1;
            }
            
        }
        my $error=0;
        if ($peakmax>1)
        {
           $error=$peakmax**0.5;
        }
        elsif($peakmax==1)
        {
           $error=$area**0.5;
        }
        elsif($peakmax==0)
        {
           $error=0;
        }
        $output{S}{$tag}{Area}=$area;
        $output{S}{$tag}{Peak_Max}=$peakmax;
        $output{S}{$tag}{Error}=$error;
    }
    return(\%output);  
    
}
#########################################################################################################
sub isotopic_correction{
    my ($area_val,$purity_value,$plex)=@_;
    my %area_hash=%{$area_val};
    my @areas;
    my @true_areas;
    my $determinant=0;
    
    if ($plex eq '8plex')
    {
        #print "$determinant\n";<>;
        $determinant = determinant_8plex($purity_value);
        #print "$determinant\n";<>;
    }
    elsif($plex eq '4plex')
    {
        $determinant = determinant_4plex($purity_value);
    }
    elsif($plex eq '6plex')
    {
         $determinant = determinant_6plex($purity_value);
    }
    elsif($plex eq '10plex')
    {
         $determinant = determinant_10plex($purity_value);
    }
    elsif($plex eq '2plex')# no known  purity correction
    {
        return(\%area_hash);
    }
    foreach my $type(sort keys %area_hash)#
    {
        foreach my $tag(sort keys %{$area_hash{$type}})
        {
            push(@areas,$area_hash{$type}{$tag}{Area})
        }
    }
    my $area;
    #print "determinant is : $determinant\n";
    #if ((scalar @areas) == 2)
    #{
    #    $area = Math::MatrixReal->new_from_rows([    [$areas[0]],
    #                                                [$areas[1]]]);
    #    my $true_area=$determinant * $area;
    #    $true_areas[0] = $true_area -> element (1,1);
    #    $true_areas[1] = $true_area -> element (2,1);
    #}
    if ((scalar @areas) == 4)
    {
        $area = Math::MatrixReal->new_from_rows([   [$areas[0]],
                                                    [$areas[1]],
                                                    [$areas[2]],
                                                    [$areas[3]]]);
        my $true_area=$determinant * $area;
        $true_areas[0] = $true_area -> element (1,1);
        $true_areas[1] = $true_area -> element (2,1);
        $true_areas[2] = $true_area -> element (3,1);
        $true_areas[3] = $true_area -> element (4,1);
    }
    elsif ((scalar @areas) == 6)
    {
        $area = Math::MatrixReal->new_from_rows([    [$areas[0]],
                                                    [$areas[1]],
                                                    [$areas[2]],
                                                    [$areas[3]],
                                                    [$areas[4]],
                                                    [$areas[5]]]);
        my $true_area=$determinant * $area;
        $true_areas[0] = $true_area -> element (1,1);
        $true_areas[1] = $true_area -> element (2,1);
        $true_areas[2] = $true_area -> element (3,1);
        $true_areas[3] = $true_area -> element (4,1);
        $true_areas[4] = $true_area -> element (5,1);
        $true_areas[5] = $true_area -> element (6,1);
    }
    elsif ((scalar @areas) == 8)
    {
        $area = Math::MatrixReal->new_from_rows([    [$areas[0]],
                                                    [$areas[1]],
                                                    [$areas[2]],
                                                    [$areas[3]],
                                                    [$areas[4]],
                                                    [$areas[5]],
                                                    [$areas[6]],
                                                    [$areas[7]]]);
        my $true_area=$determinant * $area;
        $true_areas[0] = $true_area -> element (1,1);
        $true_areas[1] = $true_area -> element (2,1);
        $true_areas[2] = $true_area -> element (3,1);
        $true_areas[3] = $true_area -> element (4,1);
        $true_areas[4] = $true_area -> element (5,1);
        $true_areas[5] = $true_area -> element (6,1);
        $true_areas[6] = $true_area -> element (7,1);
        $true_areas[7] = $true_area -> element (8,1);
    }
    elsif ((scalar @areas) == 10)
    {
        $area = Math::MatrixReal->new_from_rows([    [$areas[0]],
                                                    [$areas[1]],
                                                    [$areas[2]],
                                                    [$areas[3]],
                                                    [$areas[4]],
                                                    [$areas[5]],
                                                    [$areas[6]],
                                                    [$areas[7]],
                                                    [$areas[8]],
                                                    [$areas[9]]]);
        my $true_area=$determinant * $area;
        $true_areas[0] = $true_area -> element (1,1);
        $true_areas[1] = $true_area -> element (2,1);
        $true_areas[2] = $true_area -> element (3,1);
        $true_areas[3] = $true_area -> element (4,1);
        $true_areas[4] = $true_area -> element (5,1);
        $true_areas[5] = $true_area -> element (6,1);
        $true_areas[6] = $true_area -> element (7,1);
        $true_areas[7] = $true_area -> element (8,1);
        $true_areas[8] = $true_area -> element (9,1);
        $true_areas[9] = $true_area -> element (10,1);
    }
                
    foreach my $type(sort keys %area_hash)
    {
        foreach my $tag(sort keys %{$area_hash{$type}})
        {
            #print "$true_areas[0]";<>
	    $area_hash{$type}{$tag}{Area}=$true_areas[0];
            #print "area=$area_hash{$type}{$tag}{Area}";<>;
            shift @true_areas;    
        }
    }
    return(\%area_hash);
  
}
#########################################################################################################
sub determinant_8plex{
    my ($purity_val) = @_;
    open(PC,"$purity_val") or die "Purity_correction does not have $purity_val";
    my @arr=<PC>;
    shift@arr;
    shift@arr;
    my @temp;
    foreach my $line(@arr)
    {
        chomp $line;
        my @tem=split(/\t/,$line);
        shift @tem;
        push(@temp,@tem);
    }
    my $purity_val1=join",",@temp;
    my @purecorrs = split(/,/,$purity_val1);
    @purecorrs=@{convert_decimal(\@purecorrs)};

    my ($a113,$b113,$s,$c113,$d113,$a114,$b114,$t,$c114,$d114,$a115,$b115,$u,$c115,$d115,$a116,$b116,$v,$c116,$d116,$a117,$b117,$w,$c117,$d117,$a118,$b118,$x,$c118,$d118,$a119,$b119,$y,$c119,$d119,$a121,$b121,$z,$c121,$d121) =
        ($purecorrs[0],$purecorrs[1],$purecorrs[2],$purecorrs[3],$purecorrs[4],
         $purecorrs[5],$purecorrs[6],$purecorrs[7],$purecorrs[8],$purecorrs[9],
         $purecorrs[10],$purecorrs[11],$purecorrs[12],$purecorrs[13],$purecorrs[14],
         $purecorrs[15],$purecorrs[16],$purecorrs[17],$purecorrs[18],$purecorrs[19],
         $purecorrs[20],$purecorrs[21],$purecorrs[22],$purecorrs[23],$purecorrs[24],
         $purecorrs[25],$purecorrs[26],$purecorrs[27],$purecorrs[28],$purecorrs[29],
         $purecorrs[30],$purecorrs[31],$purecorrs[32],$purecorrs[33],$purecorrs[34],
        $purecorrs[35],$purecorrs[36],$purecorrs[37],$purecorrs[38],$purecorrs[39]);


    my $A = Math::MatrixReal->new_from_rows([ [$s,$b114,$a115,0,0,0,0,0],
                                              [$c113,$t,$b115,$a116,0,0,0,0],
                                              [$d113,$c114,$u,$b116,$a117,0,0,0],
                                              [0,$d114,$c115,$v,$b117,$a118,0,0],
                                              [0,0,$d115,$c116,$w,$b118,$a119,0],
                                              [0,0,0,$d116,$c117,$x,$b119,0],
                                              [0,0,0,0,$d117,$c118,$y,$a121],
                                              [0,0,0,0,0,0,$d119,$z] ]);
    #my $detA = $A->det();
    #print "determinant is : $detA\n";<>;
    my $detB = $A->inverse();
    return($detB)
    
}
#########################################################################################################
sub determinant_4plex{
    my ($purity_val) = @_;
    open(PC,"$purity_val") or die "Purity_correction does not have $purity_val";
    my @arr=<PC>;
    shift@arr;
    shift@arr;
    #print "@arr";<>;
    my @temp;
    foreach my $line(@arr)
    {
        chomp $line;
        my @tem=split(/\t/,$line);
        shift @tem;
        push(@temp,@tem);
    }
    my $purity_val1=join",",@temp;
    
    my @purecorrs = split(/,/,$purity_val1);
    @purecorrs=@{convert_decimal(\@purecorrs)};

    my ($a114,$b114,$t,$c114,$d114,$a115,$b115,$u,$c115,$d115,$a116,$b116,$v,$c116,$d116,$a117,$b117,$w,$c117,$d117) =
        ($purecorrs[0],$purecorrs[1],$purecorrs[2],$purecorrs[3],$purecorrs[4],
         $purecorrs[5],$purecorrs[6],$purecorrs[7],$purecorrs[8],$purecorrs[9],
         $purecorrs[10],$purecorrs[11],$purecorrs[12],$purecorrs[13],$purecorrs[14],
         $purecorrs[15],$purecorrs[16],$purecorrs[17],$purecorrs[18],$purecorrs[19]);
         


    my $A = Math::MatrixReal->new_from_rows([ [$t,$b115,$a116,0],
                                              [$c114,$u,$b116,$a117],
                                              [$d114,$c114,$v,$b117],
                                              [0,$d115,$c116,$w] ]);
    my $detB = $A->inverse();
    return($detB)
    
}
sub determinant_6plex{
    my ($purity_val) = @_;
    open(PC,"$purity_val") or die "Purity_correction does not have $purity_val";
    my @arr=<PC>;
    shift@arr;
    shift@arr;
    my @temp;
    foreach my $line(@arr)
    {
        chomp $line;
        my @tem=split(/\t/,$line);
	#print "@tem";<>;
        shift @tem;
        push(@temp,@tem);
    }
    my $purity_val1=join",",@temp;
    
    my @purecorrs = split(/,/,$purity_val1);
    #print "@purecorrs";<>;
    @purecorrs=@{convert_decimal(\@purecorrs)};


    my $A = Math::MatrixReal->new_from_rows([ [$purecorrs[3],$purecorrs[4],$purecorrs[5],$purecorrs[6],0,0],
                                            [$purecorrs[9],$purecorrs[10],$purecorrs[11],$purecorrs[12],$purecorrs[13],0],
                                    [$purecorrs[15],$purecorrs[16],$purecorrs[17],$purecorrs[18],$purecorrs[19],$purecorrs[20]],
                                    [$purecorrs[21],$purecorrs[22],$purecorrs[23],$purecorrs[24],$purecorrs[25],$purecorrs[26]],
                                            [0,$purecorrs[28],$purecorrs[29],$purecorrs[30],$purecorrs[31],$purecorrs[32]],
                                           [0,0,$purecorrs[35],$purecorrs[36],$purecorrs[37],$purecorrs[38]] ]);
    my $detB = $A->inverse();
    return($detB)
    
}
sub determinant_10plex{
    my ($purity_val) = @_;
    open(PC,"$purity_val") or die "Purity_correction folder does not have $purity_val file";
    my @arr=<PC>;
    shift@arr;
    shift@arr;
    my @temp;
    foreach my $line(@arr)
    {
        chomp $line;
        my @tem=split(/\t/,$line);
        shift @tem;
        push(@temp,@tem);
    }
    my $purity_val1=join",",@temp;
    
    my @purecorrs = split(/,/,$purity_val1);
    @purecorrs=@{convert_decimal(\@purecorrs)};

    my $A = Math::MatrixReal->new_from_rows([   [$purecorrs[2],0,$purecorrs[3],$purecorrs[4],0,0,0,0,0,0],
                                                [0,$purecorrs[7],0,$purecorrs[8],$purecorrs[9],0,0,0,0,0],
                                                [$purecorrs[11],0,$purecorrs[12],0,$purecorrs[13],$purecorrs[14],0,0,0,0],
                                                [0,$purecorrs[16],0,$purecorrs[17],0,$purecorrs[18],$purecorrs[19],0,0,0],
                                                [$purecorrs[20],0,$purecorrs[21],0,$purecorrs[22],0,$purecorrs[23],$purecorrs[24],0,0],
                                                [0,$purecorrs[25],0,$purecorrs[26],0,$purecorrs[27],0,$purecorrs[28],$purecorrs[29],0],
                                                [0,0,$purecorrs[30],0,$purecorrs[31],0,$purecorrs[32],0,$purecorrs[33],$purecorrs[34]],
                                                [0,0,0,$purecorrs[35],0,$purecorrs[36],0,$purecorrs[37],0,$purecorrs[38]],
                                                [0,0,0,0,$purecorrs[40],0,$purecorrs[41],0,$purecorrs[42],0],
                                                [0,0,0,0,0,$purecorrs[45],0,$purecorrs[46],0,$purecorrs[47]] ]);
    my $detB = $A->inverse();
    return($detB)
    
}
#########################################################################################################
sub convert_decimal{
    my ($dec)=@_;
    my @decimal=@$dec;
    my @dec_val;
    foreach my $value(@decimal)
    {
        $value=$value/100;
        push(@dec_val,$value);
    }
    return(\@dec_val);
}
#########################################################################################################
sub normalize{
    my ($area_val,$area_type)=@_;
    my %hash=%{$area_val};
    my $total_area=0;
    foreach my $tag(sort keys(%{$hash{$area_type}}))
    {
        my $area=$hash{$area_type}{$tag}{Area};
        $total_area=$total_area+$area;
    }
    if ($total_area==0)
    {
        return(\%hash);
    }
    
    foreach my $tag(sort keys(%{$hash{$area_type}}))
    {
        my $area=$hash{$area_type}{$tag}{Area};
        $area=$area/$total_area;
        $hash{$area_type}{$tag}{Area}=$area
    }
    return(\%hash);    
}
#########################################################################################################
sub intensity_cutoff{
    my ($area_val,$threshold,$area_type)=@_;
    my %hash=%{$area_val};
    foreach my $tag(sort keys(%{$hash{$area_type}}))
    {
        my $peak_max=$hash{$area_type}{$tag}{Peak_Max};
        my $Area=$hash{$area_type}{$tag}{Area};
        if ($peak_max>=$threshold&& $Area>0)
        {
            
        }
        elsif($Area<0)
        {
            $hash{$area_type}{$tag}{Area}='#BT';##not above threhold or below threshold (#BT)
            $hash{$area_type}{$tag}{Error}='0';##not above threhold area so error 0 
        }
        else
        {
            $hash{$area_type}{$tag}{Area}='#BT';##not above threhold or below threshold (#BT)
            $hash{$area_type}{$tag}{Error}='0';##not above threhold area so error 0                
                    }
        
    }
    return(\%hash);
    
}
#########################################################################################################
sub ratio_err{
    my ($area_val,$base,$area_type)=@_;
    my %hash=%{$area_val};
    #print "base:$base\t";<>;
    foreach my $tag(sort keys(%{$hash{$area_type}}))
    {
        my $num=$hash{$area_type}{$tag}{Area};#num= numerator
        my $num_err=$hash{$area_type}{$tag}{Error};
        my $den=$hash{$area_type}{$base}{Area};#den=denominator
        my $den_err=$hash{$area_type}{$base}{Error};
        #print "numerator:$num\tdenominator:$den\tnum_err:$num_err\tden_err:$den_err\n";<>;
        
        #####################################################################################
        #  Numerator     #  BT      0       >0
        ###########################################################
        # Denom     BT   #  BT      BT      100
        #           0    #  BT      BT      100
        #           >0   #  0.01    0.01    Value
        ####################################################
        
        if ($num eq '#BT' || $den eq '#BT')
        {
    
            if ($num eq '#BT' && $den eq '#BT')# If both are BT then only ratio is BT
            {
                $hash{$area_type}{$tag}{Ratio}= "";
                $hash{$area_type}{$tag}{Ratio_err}="NA";
            }
            elsif($num eq '#BT' && $den <= 0)
            {
                $hash{$area_type}{$tag}{Ratio}= "";
                $hash{$area_type}{$tag}{Ratio_err}="NA"; 
            }
            elsif($den eq '#BT' && $num <= 0)
            {
                $hash{$area_type}{$tag}{Ratio}= "";
                $hash{$area_type}{$tag}{Ratio_err}="NA";                 
            }
            elsif($num eq '#BT' && $den > 0)
            {
                $hash{$area_type}{$tag}{Ratio}= "0.01";
                $hash{$area_type}{$tag}{Ratio_err}="NA";                 
            }
            elsif($den eq '#BT' && $num > 0)
            {
                $hash{$area_type}{$tag}{Ratio}= "100";
                $hash{$area_type}{$tag}{Ratio_err}="NA"; 
            }
        }
        elsif ($num <= 0 && $den > 0)
        {
            $hash{$area_type}{$tag}{Ratio}= "0.01";
            $hash{$area_type}{$tag}{Ratio_err}="NA";
        }
        elsif ($num > 0 &&  $den <= 0)
        {
            $hash{$area_type}{$tag}{Ratio}= "100";
            $hash{$area_type}{$tag}{Ratio_err}="NA";
        }
        elsif ($num > 0 && $den > 0)
        {
            $hash{$area_type}{$tag}{Ratio}= ($num / $den);
            $hash{$area_type}{$tag}{Ratio_err}=(((($num_err/$num)**2)+(($den_err/$den)**2))**0.5)*100;##Protein pilot's %error
            if ($hash{$area_type}{$tag}{Ratio}>100)
            {
                $hash{$area_type}{$tag}{Ratio}="100";
            }
            if ($hash{$area_type}{$tag}{Ratio}<0.01)
            {
                $hash{$area_type}{$tag}{Ratio}="0.01";
            }
            
        }
        elsif($num <= 0 && $den <= 0)
        {
            $hash{$area_type}{$tag}{Ratio}= "0";
            $hash{$area_type}{$tag}{Ratio_err}="0";
        }
        else
        {
            $hash{$area_type}{$tag}{Ratio}= "NA";
            $hash{$area_type}{$tag}{Ratio_err}="NA";
        }
        
    }
    return(\%hash);
}
#########################################################################################################

1;