#!/usr/bin/perl -w
use strict;
use warnings;
use Math::MatrixReal;
use lib 'MODULES/';
use IQ;
use mgfQuant;
use mzMLQuant;
use ConfigReader;
######################################################################################################
# Author: Suruchi Aggarwal (SA)
# Thu Dec 31 11:08:20 2015
# Updated 30 July 2016 SA & AKY
# Updated Mon Aug 29 15:10:22 2016 SA
######################################################################################################
my $parameter=shift;#parameter file
if (!defined $parameter || $parameter eq "")
{
    print "Parameter File Missing\n";
    print "USAGE:- perl Quantwiz_IQ.pl Parameter_file\n";
    exit;
}

######################################################################################################
#create hash for label headers of each label method and its plexing
######################################################################################################
my %plex;#exact masses for searching peaks
my %header;#masses for headers printing

$header{iTRAQ}{'4plex'}=[114,115,116,117];
$header{iTRAQ}{'8plex'}=[113,114,115,116,117,118,119,121];
$header{TMT}{'2plex'}=[126,127];
$header{TMT}{'6plex'}=[126,127,128,129,130,131];
$header{TMT}{'10plex'}=[126,"127C","127N","128C","128N","129C","129N","130C","130N",131];

$plex{iTRAQ}{'4plex'}=[114.1112,115.1083,116.1116,117.1150];
$plex{iTRAQ}{'8plex'}=[113.1078734,114.1112318,115.1082668,116.1116252,117.1149836,118.1120185,119.1153769,121.1220937];
$plex{TMT}{'2plex'}=[126.127725,127.131079];
$plex{TMT}{'6plex'}=[126.127725,127.124760,128.134433,129.131468,130.141141,131.138176];
$plex{TMT}{'10plex'}=[126.12772591,127.12476080,127.13108075,128.12811564,128.13443559,129.13147048,129.13779043,130.13482532,130.14114527,131.13818016]; #http://pubs.acs.org/doi/full/10.1021/ac500140s

######################################################################################################
# defining a hash to mention T/B/S area method on command line
my %area_name;
$area_name{T}="Trapezoid method";
$area_name{B}="Sum intensity based baseline corrected method";
$area_name{S}="Sum intensity method";

######################################################################################################
#reading config
my %params=%{conf_read($parameter)};#refer below for structure of hash (params)
my($pREF,$eREF)=check_params(\%params,\%plex,\%header); #params hash and error message
%params=%$pREF;
my @err_msg=@$eREF;
if (scalar @err_msg >0)
{
    print join "\n",@err_msg;
    print "\nPlease correct the above errors and run the program again\n";
    exit;
}

#####################################################################################################
#Structure of hash params
#####################################################################################################
#Spectra_Folder=
#Input_file_extension=  not being used in this code
#Label_type=
#Plex=
#min_reporters=
#Spectrum_area=
#Base_reporter=
#Normalization=
#Peak_intensity_threshold=
#Tol_unit
#Tol_min=
#Tol_max=
#Purity_corrections_values=
######################################################################################################
################################################################################################################################
if ($params{Input_file_extension} eq "mgf")
{
    mgf_quant($params{Spectra_Folder},\%params,\%header,\%plex,\%area_name);
}
elsif($params{Input_file_extension} eq "mzML")
{
    mzML_quant($params{Spectra_Folder},\%params,\%header,\%plex,\%area_name);
}


#####################################################################################################


exit;

